﻿/*
www.E2CRE8.be - Brainbox Arduino  - by Bart Huyskens
04/2016 - ATMEL STUDIO 7

This program demonstrates how Atmel Studio 7 can be used to program the Brainbox Arduino - based on the ARduino Leonardo
Look at the manual on www.E2CRE8.be on how to set up STUDIO7 and AVRDUDE to program the Hardware of the BBA

This program  demonstrates how the Analog inputs channels can be programmed in "C"
Use the datasheet of the ATMEGA32U4 from page 298 for more info

We have written 2 functions

READ_ADC_BYTE_CHANNEL: converts the Analog input into a 8 bit digital value - tested for channel 0-7
READ_ADC_INT_CHANNEL: converts the Analog input into a 10 bit digital value - tested for channel 0-7

Connect a potmeter to ADC7 (AN0 ARduino)
to demonstrate the result of the AD conversion is used to change the frequency at the buzzer - connected to E6 on the BBA
*/
#include <Arduino.h>		// by including this lib - the usage of most ARduino libraries is made possible - not necessary if you are not using any arduino instructions
#include <avr/io.h>			// This lib is needed for the C compiler to know the correct addresses for registers like DDRB, PORTB, ...
#include <util/delay.h>		// This lib is needed to calculate the correct delays for "_delay_ms(10);" 
//- the clockspeed of the ARduino Leonardo is automatically set when opening an Arduino project under STUDIO 7

char ADRESULT = 0;
int PERIOD = 0;

unsigned int READ_ADC_INT_CHANNEL( unsigned char channel ); // function prototype
unsigned char READ_ADC_BYTE_CHANNEL( unsigned char channel ); // function prototype


int main (void)			// main program
{
  DDRF = 0b00000000; // pin F0 (ARduino pin 23) is digital input (all pins of portF are digital inputs...)
  DDRE = 0b01000000; // pin E6  (ARduino pin 7) connected to buzzer = output pin

  
    while(1)				// while(1) is an ever lasting loop
		{
		ADRESULT = READ_ADC_BYTE_CHANNEL(7); // convert the analog signal of channel 7 (ARduino A0) into a 8 bit digital signal and store it into "ADRESULT"
			
				PORTE = 0b01000000;			// make pin E6 high
				for (char x = ADRESULT; x>0; x--) // make pin E6 high for as many usec as ADRESULT
				{
				_delay_us(1);	
				}
							//wait
				PORTE = 0b00000000;			// make pin E6 low
				for (char x = ADRESULT; x>0; x--) // // make pin E6 hlow for as many usec as ADRESULT
				{
				_delay_us(1);
				}
	   
		}
}


/*************************************************************************
ADC FUNCTIONS - ADC FUNCTIONS - ADC FUNCTIONS - ADC FUNCTIONS
**************************************************************************/

// returns a 10 bit result from an AD conversion from a certain ADC channel
//Channels 0-7
// A 10 bit result needs to be loaded in an integer variable
unsigned int READ_ADC_INT_CHANNEL( unsigned char channel ){
	
	DDRF &= ~(1<<channel); // config selected channel as input

	// Right adjust + choice of channel
	ADMUX = 0b01000000 +  channel;
	
	// Activate ADC - Stop conversion - prescaler 128 (for ADC 50K<..<200k) (16Mhz / 128 = 125000)
	ADCSRA = 0b10000111;
	
	// start conversion and wait for completion
	ADCSRA |= (1<<ADSC);
	while (ADCSRA & (1<<ADSC));
	
	// return A/D conversion result
	unsigned char ADCLBuff = ADCL; // first read ADCL!!
	return ( ((unsigned int) ADCH ) << 8 ) + ADCLBuff;
}

// returns a 8 bit result from an AD conversion from a certain ADC channel
//Channels 0-17
// A 8 bit result needs to be loaded in a BYTE variable
unsigned char READ_ADC_BYTE_CHANNEL( unsigned char channel ){
	
	DDRF &= ~(1<<channel); // config selected channel as input

	// Left adjust + choice of channel
	ADMUX = 0b01100000 +  channel ;
	
	// Activate ADC - Stop conversion - prescaler 128 (for ADC 50K<..<200k) (16Mhz / 128 = 125000)
	ADCSRA = 0b10000111;
	
	// start conversion and wait for completion
	ADCSRA |=(1<<ADSC);
	while (ADCSRA & (1<<ADSC));
	
	// return A/D conversion result
	return ADCH;
}
